##Copyright 2011-2017 Thomas Paviot (tpaviot@gmail.com)
##
##This file is part of pythonOCC.
##
##pythonOCC is free software: you can redistribute it and/or modify
##it under the terms of the GNU Lesser General Public License as published by
##the Free Software Foundation, either version 3 of the License, or
##(at your option) any later version.
##
##pythonOCC is distributed in the hope that it will be useful,
##but WITHOUT ANY WARRANTY; without even the implied warranty of
##MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##GNU Lesser General Public License for more details.
##
##You should have received a copy of the GNU Lesser General Public License
##along with pythonOCC.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import tempfile
import uuid
from xml.etree import ElementTree

from OCC.Core.Tesselator import ShapeTesselator
from OCC import VERSION as OCC_VERSION

from OCC.Extend.TopologyUtils import is_edge, is_wire, discretize_edge, discretize_wire
from OCC.Display.WebGl.simple_server import start_server

def spinning_cursor():
    while True:
        for cursor in '|/-\\':
            yield cursor

X3DFILE_HEADER = """<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE X3D PUBLIC "ISO//Web3D//DTD X3D 3.3//EN" "http://www.web3d.org/specifications/x3d-3.3.dtd">
<X3D profile="Immersive" version="3.3" xmlns:xsd="http://www.w3.org/2001/XMLSchema-instance" xsd:noNamespaceSchemaLocation="http://www.web3d.org/specifications/x3d-3.3.xsd">
<head>
    <meta name="generator" content="pythonocc-%s X3D exporter (www.pythonocc.org)"/>
    <meta name="creator" content="pythonocc-%s generator"/>
    <meta name="identifier" content="http://www.pythonocc.org"/>
    <meta name="description" content="pythonocc-%s x3dom based shape rendering"/>
</head>
<Scene>
""" % (OCC_VERSION, OCC_VERSION, OCC_VERSION)

HEADER = """
<head>
    <title>pythonOCC @VERSION@ x3dom renderer</title>
    <meta name='Author' content='Thomas Paviot - tpaviot@gmail.com'>
    <meta name='Keywords' content='WebGl,pythonOCC'>
    <meta charset="utf-8">
    <link rel="stylesheet" type="text/css" href="https://x3dom.org/release/x3dom.css">
    <script src="https://x3dom.org/release/x3dom.js"></script>
    <style>
        body {
            background: linear-gradient(@bg_gradient_color1@, @bg_gradient_color2@);
            margin: 0px;
            overflow: hidden;
        }
        #pythonocc_rocks {
            padding: 5px;
            position: absolute;
            left: 1%;
            bottom: 2%;
            height: 38px;
            width: 280px;
            border-radius: 5px;
            border: 2px solid #f7941e;
            opacity: 0.7;
            font-family: Arial;
            background-color: #414042;
            color: #ffffff;
            font-size: 14px;
            opacity: 0.5;
        }
        #commands {
            padding: 5px;
            position: absolute;
            right: 1%;
            top: 2%;
            height: 65px;
            width: 180px;
            border-radius: 5px;
            border: 2px solid #f7941e;
            opacity: 0.7;
            font-family: Arial;
            background-color: #414042;
            color: #ffffff;
            font-size: 14px;
            opacity: 0.5;
        }
        a {
            color: #f7941e;
            text-decoration: none;
        }
        a:hover {
            color: #ffffff;
        }
    </style>
</head>
"""

BODY = """
<body>
    @X3DSCENE@
    <div id="pythonocc_rocks">
        pythonocc-@VERSION@ <a href="https://www.x3dom.org" target="_blank">x3dom</a> renderer
        <br>Check our blog at
        <a href=http://www.pythonocc.org>http://www.pythonocc.org</a>
    </div>
    <div id="commands">
    <b>t</b> view/hide shape<br>
    <b>r</b> reset view<br>
    <b>a</b> show all<br>
    <b>u</b> upright<br>
    </div>
    <script>
    var selected_target_color = null;
    var current_selected_shape = null;
    var current_mat = null;
    function fitCamera()
    {            
        var x3dElem = document.getElementById('pythonocc-x3d-scene');            
        x3dElem.runtime.fitAll();
    }
    function select(the_shape) // called whenever a shape is clicked
    {
        // restore color for previous selected shape
        if (current_mat) {
            current_mat.diffuseColor = selected_target_color;
        }
        // store the shape for future process
        current_selected_shape = the_shape;
        console.log(the_shape);
        // store color, to be restored later
        appear = current_selected_shape.getElementsByTagName("Appearance")[0];
        mat = appear.getElementsByTagName("Material")[0];
        current_mat = mat;
        console.log(mat);
        selected_target_color = mat.diffuseColor;
        mat.diffuseColor = "1, 0.65, 0";
        //console.log(the_shape.getElementsByTagName("Appearance"));//.getAttribute('diffuseColor'));
    }
    function onDocumentKeyPress(event) {
      event.preventDefault();
      if (event.key=="t") {  // t key
         if (current_selected_shape) {
           if (current_selected_shape.render == "true") {
              current_selected_shape.render = "false";
           }
           else {
              current_selected_shape.render = "true";
           }
         }
      }

    }
    // add events
    document.addEventListener('keypress', onDocumentKeyPress, false);
    </script>
</body>
"""


def export_edge_to_indexed_lineset(edge_point_set):
    str_x3d_to_return = "\t<LineSet vertexCount='%i'>" % len(edge_point_set)
    str_x3d_to_return += "<Coordinate point='"
    for p in edge_point_set:
        str_x3d_to_return += "%g %g %g " % (p[0], p[1], p[2])
    str_x3d_to_return += "'/></LineSet>\n"
    return str_x3d_to_return


def indexed_lineset_to_x3d_string(str_linesets, header=True, footer=True, ils_id=0):
    """ takes an str_lineset, coming for instance from export_curve_to_ils,
    and export to an X3D string"""
    if header:
        x3dfile_str = X3DFILE_HEADER
    else:
        x3dfile_str = ""
    x3dfile_str += "<Switch whichChoice='0' id='swBRP'>\n"
    x3dfile_str += "\t<Group>\n"

    ils_id = 0
    for str_lineset in str_linesets:
        x3dfile_str += "\t\t<Transform scale='1 1 1'><Shape DEF='edg%s'>\n" % ils_id
        # empty appearance, but the x3d validator complains if nothing set
        x3dfile_str += "\t\t\t<Appearance><Material emissiveColor='0 0 0'/></Appearance>\n\t\t"
        x3dfile_str += str_lineset
        x3dfile_str += "\t\t</Shape></Transform>\n"
        ils_id += 1

    x3dfile_str += "\t</Group>\n"
    x3dfile_str += "</Switch>\n"
    if footer:
        x3dfile_str += '</Scene>\n</X3D>\n'

    return x3dfile_str


class HTMLHeader:
    def __init__(self, bg_gradient_color1="#ced7de", bg_gradient_color2="#808080"):
        self._bg_gradient_color1 = bg_gradient_color1
        self._bg_gradient_color2 = bg_gradient_color2


    def get_str(self):
        header_str = HEADER.replace('@bg_gradient_color1@', '%s' % self._bg_gradient_color1)
        header_str = header_str.replace('@bg_gradient_color2@', '%s' % self._bg_gradient_color2)
        header_str = header_str.replace('@VERSION@', OCC_VERSION)
        return header_str


class HTMLBody:
    def __init__(self, x3d_shapes, axes_plane, axes_plane_zoom_factor=1.):
        """ x3d_shapes is a list that contains uid for each shape
        """
        self._x3d_shapes = x3d_shapes
        self.spinning_cursor = spinning_cursor()
        self._display_axes_plane = axes_plane
        self._axis_plane_zoom_factor = axes_plane_zoom_factor

    def get_str(self):
        # get the location where pythonocc is running from
        body_str = BODY.replace('@VERSION@', OCC_VERSION)
        x3dcontent = '\n\t<x3d id="pythonocc-x3d-scene" style="width:100%;border: none" >\n\t\t<Scene>\n'
        nb_shape = len(self._x3d_shapes)
        cur_shp = 1
        if self._display_axes_plane:
            x3dcontent += """
            <transform scale="%g,%g,%g">
            <transform id="plane_smallaxe_Id" rotation="1 0 0 -1.57079632679">
                <inline url="https://rawcdn.githack.com/x3dom/component-editor/master/static/x3d/plane.x3d" mapDEFToID="true" namespaceName="plane"></inline>
                <inline url="https://rawcdn.githack.com/x3dom/component-editor/master/static/x3d/axesSmall.x3d" mapDEFToID="true" namespaceName="axesSmall"></inline>
            </transform>
            <inline url="https://rawcdn.githack.com/x3dom/component-editor/master/static/x3d/axes.x3d" mapDEFToID="true" namespaceName="axes"></inline>
            </transform>
            """ % (self._axis_plane_zoom_factor, self._axis_plane_zoom_factor, self._axis_plane_zoom_factor)
            # global rotateso that z is aligne properly
        x3dcontent += '<transform id="glbal_scene_rotation_Id" rotation="1 0 0 -1.57079632679">'
        for shp_uid in self._x3d_shapes:
            sys.stdout.write("\r%s meshing shapes... %i%%" % (next(self.spinning_cursor),
                                                              round(cur_shp / nb_shape * 100)))
            sys.stdout.flush()

            x3dcontent += '\t\t\t<Inline onload="fitCamera()" mapDEFToID="true" url="%s.x3d"></Inline>\n' % shp_uid
            cur_shp += 1
        x3dcontent += '</transform>'
        x3dcontent += "\t\t</Scene>\n\t</x3d>\n"
        body_str = body_str.replace('@X3DSCENE@', x3dcontent)
        return body_str


class X3DExporter:
    """ A class for exporting a TopoDS_Shape to an x3d file """
    def __init__(self,
                 shape,  # the TopoDS shape to mesh
                 vertex_shader,  # the vertex_shader, passed as a string
                 fragment_shader,  # the fragment shader, passed as a string
                 export_edges,  # if yes, edges are exported to IndexedLineSet (might be SLOWW)
                 color,  # the default shape color
                 specular_color,  # shape specular color (white by default)
                 shininess,  # shape shininess
                 transparency,  # shape transparency
                 line_color,  # edge color
                 line_width,  # edge liewidth,
                 mesh_quality  # mesh quality default is 1., good is <1, bad is >1
                ):
        self._shape = shape
        self._vs = vertex_shader
        self._fs = fragment_shader
        self._export_edges = export_edges
        self._color = color
        self._shininess = shininess
        self._specular_color = specular_color
        self._transparency = transparency
        self._mesh_quality = mesh_quality
        # the list of indexed face sets that compose the shape
        # if ever the map_faces_to_mesh option is enabled, this list
        # maybe composed of dozains of TriangleSet
        self._triangle_sets = []
        self._line_sets = []
        self._x3d_string = ""  # the string that contains the x3d description

    def compute(self):
        shape_tesselator = ShapeTesselator(self._shape)
        shape_tesselator.Compute(compute_edges=self._export_edges,
                                 mesh_quality=self._mesh_quality,
                                 parallel=True)
        self._triangle_sets.append(shape_tesselator.ExportShapeToX3DTriangleSet())
        # then process edges
        if self._export_edges:
            # get number of edges
            nbr_edges = shape_tesselator.ObjGetEdgeCount()
            for i_edge in range(nbr_edges):
                edge_point_set = []
                nbr_vertices = shape_tesselator.ObjEdgeGetVertexCount(i_edge)
                for i_vert in range(nbr_vertices):
                    edge_point_set.append(shape_tesselator.GetEdgeVertex(i_edge, i_vert))
                ils = export_edge_to_indexed_lineset(edge_point_set)
                self._line_sets.append(ils)

    def to_x3dfile_string(self, shape_id):
        x3dfile_str = X3DFILE_HEADER
        for triangle_set in self._triangle_sets:
            x3dfile_str += "<Switch whichChoice='0' id='swBRP'><Transform scale='1 1 1'><Shape DEF='shape%i' onclick='" % shape_id
            x3dfile_str += "select(this);"
            x3dfile_str += "'><Appearance>\n"
            #
            # set Material or shader
            #
            if self._vs is None and self._fs is None:
                x3dfile_str += "<Material id='color' diffuseColor="
                x3dfile_str += "'%g %g %g'" % (self._color[0], self._color[1], self._color[2])
                x3dfile_str += " shininess="
                x3dfile_str += "'%g'" % self._shininess
                x3dfile_str += " specularColor="
                x3dfile_str += "'%g %g %g'" % (self._specular_color[0], self._specular_color[1], self._specular_color[2])
                x3dfile_str += " transparency='%g'>\n" % self._transparency
                x3dfile_str += "</Material>\n"
            else:  # set shaders
                x3dfile_str += '<ComposedShader><ShaderPart type="VERTEX" style="display:none;">\n'
                x3dfile_str += self._vs
                x3dfile_str += '</ShaderPart>\n'
                x3dfile_str += '<ShaderPart type="FRAGMENT" style="display:none;">\n'
                x3dfile_str += self._fs
                x3dfile_str += '</ShaderPart></ComposedShader>\n'
            x3dfile_str += '</Appearance>\n'
            # export triangles
            x3dfile_str += triangle_set
            x3dfile_str += "</Shape></Transform></Switch>\n"
        # and now, process edges
        if self._export_edges:
            # below '0' means show all
            # -1 means doesn't show line
            # the "Switch" node selects the group to be displayed

            x3dfile_str += indexed_lineset_to_x3d_string(self._line_sets, header=False, footer=False)
        x3dfile_str += '</Scene>\n</X3D>\n'

        #
        # use ElementTree to ensure xml file quality
        #
        xml_et = ElementTree.fromstring(x3dfile_str)
        clean_x3d_str = ElementTree.tostring(xml_et, encoding='utf8').decode('utf8')

        return clean_x3d_str

    def write_to_file(self, filename, shape_id):
        with open(filename, "w") as f:
            f.write(self.to_x3dfile_string(shape_id))


class X3DomRenderer:
    def __init__(self, path=None, display_axes_plane=True, axes_plane_zoom_factor=1.):
        if not path:  # by default, write to a temp directory
            self._path = tempfile.mkdtemp()
        else:
            self._path = path
        self._html_filename = os.path.join(self._path, 'index.html')
        self._x3d_shapes = {}
        self._x3d_edges = {}
        self._axes_plane = display_axes_plane  # display the small RVB axes and the plane
        self._axes_plane_zoom_factor = axes_plane_zoom_factor

        print("## x3dom webgl renderer - render axes/planes : %r - axes/plane zoom factor : %g" % (self._axes_plane,
                                                                                                   self._axes_plane_zoom_factor))

    def DisplayShape(self,
                     shape,
                     vertex_shader=None,
                     fragment_shader=None,
                     export_edges=False,
                     color=(0.65, 0.65, 0.7),
                     specular_color=(0.2, 0.2, 0.2),
                     shininess=0.9,
                     transparency=0.,
                     line_color=(0, 0., 0.),
                     line_width=2.,
                     mesh_quality=1.):
        """ Adds a shape to the rendering buffer. This class computes the x3d file
        """
        # if the shape is an edge or a wire, use the related functions
        if is_edge(shape):
            print("X3D exporter, discretize an edge")
            pnts = discretize_edge(shape)
            edge_hash = "edg%s" % uuid.uuid4().hex
            line_set = export_edge_to_indexed_lineset(pnts)
            x3dfile_content = indexed_lineset_to_x3d_string([line_set], ils_id=edge_hash)
            edge_full_path = os.path.join(self._path, edge_hash + '.x3d')
            with open(edge_full_path, "w") as edge_file:
                edge_file.write(x3dfile_content)
            # store this edge hash
            self._x3d_edges[edge_hash] = [color, line_width]
            return self._x3d_shapes, self._x3d_edges

        if is_wire(shape):
            print("X3D exporter, discretize a wire")
            pnts = discretize_wire(shape)
            wire_hash = "wir%s" % uuid.uuid4().hex
            line_set = export_edge_to_indexed_lineset(pnts)
            x3dfile_content = indexed_lineset_to_x3d_string([line_set], ils_id=wire_hash)
            wire_full_path = os.path.join(self._path, wire_hash + '.x3d')
            with open(wire_full_path, "w") as wire_file:
                wire_file.write(x3dfile_content)
            # store this edge hash
            self._x3d_edges[wire_hash] = [color, line_width]
            return self._x3d_shapes, self._x3d_edges

        shape_uuid = uuid.uuid4().hex
        shape_hash = "shp%s" % shape_uuid
        x3d_exporter = X3DExporter(shape, vertex_shader, fragment_shader,
                                   export_edges, color,
                                   specular_color, shininess, transparency,
                                   line_color, line_width, mesh_quality)
        x3d_exporter.compute()
        x3d_filename = os.path.join(self._path, "%s.x3d" % shape_hash)
        # the x3d filename is computed from the shape hash
        shape_id = len(self._x3d_shapes)
        x3d_exporter.write_to_file(x3d_filename, shape_id)

        self._x3d_shapes[shape_hash] = [export_edges, color, specular_color, shininess,
                                        transparency, line_color, line_width]
        return self._x3d_shapes, self._x3d_edges

    def render(self, addr="localhost", server_port=8080, open_webbrowser=False):
        """ Call the render() method to display the X3D scene.
        """
        # first generate the HTML root file
        self.generate_html_file(self._axes_plane, self._axes_plane_zoom_factor)
        # then create a simple web server
        start_server(addr, server_port, self._path, open_webbrowser)

    def generate_html_file(self, axes_plane, axes_plane_zoom_factor):
        """ Generate the HTML file to be rendered wy the web browser
        axes_plane: a boolean, telles wether or not display axes
        """
        with open(self._html_filename, "w") as html_file:
            html_file.write("<!DOCTYPE HTML>\n")
            html_file.write('<html lang="en">')
            # header
            html_file.write(HTMLHeader().get_str())
            # body
            # merge shapes and edges keys
            all_shapes = list(self._x3d_shapes) + list(self._x3d_edges)
            html_file.write(HTMLBody(all_shapes, axes_plane, axes_plane_zoom_factor).get_str())
            html_file.write("</html>\n")



from enum import IntEnum
from typing import overload, NewType, Optional, Tuple

from OCC.Core.Standard import *
from OCC.Core.NCollection import *
from OCC.Core.TCollection import *
from OCC.Core.StepData import *
from OCC.Core.StepBasic import *


class StepRepr_Array1OfMaterialPropertyRepresentation:
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, theLower: int, theUpper: int) -> None: ...
    def __getitem__(self, index: int) -> False: ...
    def __setitem__(self, index: int, value: False) -> None: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[False]: ...
    def next(self) -> False: ...
    __next__ = next
    def Init(self, theValue: False) -> None: ...
    def Size(self) -> int: ...
    def Length(self) -> int: ...
    def IsEmpty(self) -> bool: ...
    def Lower(self) -> int: ...
    def Upper(self) -> int: ...
    def IsDetectable(self) -> bool: ...
    def IsAllocated(self) -> bool: ...
    def First(self) -> False: ...
    def Last(self) -> False: ...
    def Value(self, theIndex: int) -> False: ...
    def SetValue(self, theIndex: int, theValue: False) -> None: ...

class StepRepr_Array1OfPropertyDefinitionRepresentation:
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, theLower: int, theUpper: int) -> None: ...
    def __getitem__(self, index: int) -> False: ...
    def __setitem__(self, index: int, value: False) -> None: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[False]: ...
    def next(self) -> False: ...
    __next__ = next
    def Init(self, theValue: False) -> None: ...
    def Size(self) -> int: ...
    def Length(self) -> int: ...
    def IsEmpty(self) -> bool: ...
    def Lower(self) -> int: ...
    def Upper(self) -> int: ...
    def IsDetectable(self) -> bool: ...
    def IsAllocated(self) -> bool: ...
    def First(self) -> False: ...
    def Last(self) -> False: ...
    def Value(self, theIndex: int) -> False: ...
    def SetValue(self, theIndex: int, theValue: False) -> None: ...

class StepRepr_Array1OfRepresentationItem:
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, theLower: int, theUpper: int) -> None: ...
    def __getitem__(self, index: int) -> False: ...
    def __setitem__(self, index: int, value: False) -> None: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[False]: ...
    def next(self) -> False: ...
    __next__ = next
    def Init(self, theValue: False) -> None: ...
    def Size(self) -> int: ...
    def Length(self) -> int: ...
    def IsEmpty(self) -> bool: ...
    def Lower(self) -> int: ...
    def Upper(self) -> int: ...
    def IsDetectable(self) -> bool: ...
    def IsAllocated(self) -> bool: ...
    def First(self) -> False: ...
    def Last(self) -> False: ...
    def Value(self, theIndex: int) -> False: ...
    def SetValue(self, theIndex: int, theValue: False) -> None: ...

class StepRepr_Array1OfShapeAspect:
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, theLower: int, theUpper: int) -> None: ...
    def __getitem__(self, index: int) -> False: ...
    def __setitem__(self, index: int, value: False) -> None: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[False]: ...
    def next(self) -> False: ...
    __next__ = next
    def Init(self, theValue: False) -> None: ...
    def Size(self) -> int: ...
    def Length(self) -> int: ...
    def IsEmpty(self) -> bool: ...
    def Lower(self) -> int: ...
    def Upper(self) -> int: ...
    def IsDetectable(self) -> bool: ...
    def IsAllocated(self) -> bool: ...
    def First(self) -> False: ...
    def Last(self) -> False: ...
    def Value(self, theIndex: int) -> False: ...
    def SetValue(self, theIndex: int, theValue: False) -> None: ...

class StepRepr_SequenceOfMaterialPropertyRepresentation:
    def __init__(self) -> None: ...
    def __len__(self) -> int: ...
    def Size(self) -> int: ...
    def Clear(self) -> None: ...
    def First(self) -> False: ...
    def Last(self) -> False: ...
    def Length(self) -> int: ...
    def Append(self, theItem: False) -> False: ...
    def Prepend(self, theItem: False) -> False: ...
    def RemoveFirst(self) -> None: ...
    def Reverse(self) -> None: ...
    def Value(self, theIndex: int) -> False: ...
    def SetValue(self, theIndex: int, theValue: False) -> None: ...

class StepRepr_SequenceOfRepresentationItem:
    def __init__(self) -> None: ...
    def __len__(self) -> int: ...
    def Size(self) -> int: ...
    def Clear(self) -> None: ...
    def First(self) -> False: ...
    def Last(self) -> False: ...
    def Length(self) -> int: ...
    def Append(self, theItem: False) -> False: ...
    def Prepend(self, theItem: False) -> False: ...
    def RemoveFirst(self) -> None: ...
    def Reverse(self) -> None: ...
    def Value(self, theIndex: int) -> False: ...
    def SetValue(self, theIndex: int, theValue: False) -> None: ...

class StepRepr_AssemblyComponentUsageSubstitute(Standard_Transient):
	def __init__(self) -> None: ...
	def Base(self) -> StepRepr_AssemblyComponentUsage: ...
	def Definition(self) -> TCollection_HAsciiString: ...
	def Init(self, aName: TCollection_HAsciiString, aDef: TCollection_HAsciiString, aBase: StepRepr_AssemblyComponentUsage, aSubs: StepRepr_AssemblyComponentUsage) -> None: ...
	def Name(self) -> TCollection_HAsciiString: ...
	def SetBase(self, aBase: StepRepr_AssemblyComponentUsage) -> None: ...
	def SetDefinition(self, aDef: TCollection_HAsciiString) -> None: ...
	def SetName(self, aName: TCollection_HAsciiString) -> None: ...
	def SetSubstitute(self, aSubstitute: StepRepr_AssemblyComponentUsage) -> None: ...
	def Substitute(self) -> StepRepr_AssemblyComponentUsage: ...

class StepRepr_CharacterizedDefinition(StepData_SelectType):
	def __init__(self) -> None: ...
	def CaseNum(self, ent: Standard_Transient) -> int: ...
	def CharacterizedObject(self) -> StepBasic_CharacterizedObject: ...
	def DocumentFile(self) -> StepBasic_DocumentFile: ...
	def ProductDefinition(self) -> StepBasic_ProductDefinition: ...
	def ProductDefinitionRelationship(self) -> StepBasic_ProductDefinitionRelationship: ...
	def ProductDefinitionShape(self) -> StepRepr_ProductDefinitionShape: ...
	def ShapeAspect(self) -> StepRepr_ShapeAspect: ...
	def ShapeAspectRelationship(self) -> StepRepr_ShapeAspectRelationship: ...

class StepRepr_ConfigurationDesign(Standard_Transient):
	def __init__(self) -> None: ...
	def Configuration(self) -> StepRepr_ConfigurationItem: ...
	def Design(self) -> StepRepr_ConfigurationDesignItem: ...
	def Init(self, aConfiguration: StepRepr_ConfigurationItem, aDesign: StepRepr_ConfigurationDesignItem) -> None: ...
	def SetConfiguration(self, Configuration: StepRepr_ConfigurationItem) -> None: ...
	def SetDesign(self, Design: StepRepr_ConfigurationDesignItem) -> None: ...

class StepRepr_ConfigurationDesignItem(StepData_SelectType):
	def __init__(self) -> None: ...
	def CaseNum(self, ent: Standard_Transient) -> int: ...
	def ProductDefinition(self) -> StepBasic_ProductDefinition: ...
	def ProductDefinitionFormation(self) -> StepBasic_ProductDefinitionFormation: ...

class StepRepr_ConfigurationEffectivity(StepBasic_ProductDefinitionEffectivity):
	def __init__(self) -> None: ...
	def Configuration(self) -> StepRepr_ConfigurationDesign: ...
	def Init(self, aEffectivity_Id: TCollection_HAsciiString, aProductDefinitionEffectivity_Usage: StepBasic_ProductDefinitionRelationship, aConfiguration: StepRepr_ConfigurationDesign) -> None: ...
	def SetConfiguration(self, Configuration: StepRepr_ConfigurationDesign) -> None: ...

class StepRepr_ConfigurationItem(Standard_Transient):
	def __init__(self) -> None: ...
	def Description(self) -> TCollection_HAsciiString: ...
	def HasDescription(self) -> bool: ...
	def HasPurpose(self) -> bool: ...
	def Id(self) -> TCollection_HAsciiString: ...
	def Init(self, aId: TCollection_HAsciiString, aName: TCollection_HAsciiString, hasDescription: bool, aDescription: TCollection_HAsciiString, aItemConcept: StepRepr_ProductConcept, hasPurpose: bool, aPurpose: TCollection_HAsciiString) -> None: ...
	def ItemConcept(self) -> StepRepr_ProductConcept: ...
	def Name(self) -> TCollection_HAsciiString: ...
	def Purpose(self) -> TCollection_HAsciiString: ...
	def SetDescription(self, Description: TCollection_HAsciiString) -> None: ...
	def SetId(self, Id: TCollection_HAsciiString) -> None: ...
	def SetItemConcept(self, ItemConcept: StepRepr_ProductConcept) -> None: ...
	def SetName(self, Name: TCollection_HAsciiString) -> None: ...
	def SetPurpose(self, Purpose: TCollection_HAsciiString) -> None: ...

class StepRepr_DataEnvironment(Standard_Transient):
	def __init__(self) -> None: ...
	def Description(self) -> TCollection_HAsciiString: ...
	def Elements(self) -> StepRepr_HArray1OfPropertyDefinitionRepresentation: ...
	def Init(self, aName: TCollection_HAsciiString, aDescription: TCollection_HAsciiString, aElements: StepRepr_HArray1OfPropertyDefinitionRepresentation) -> None: ...
	def Name(self) -> TCollection_HAsciiString: ...
	def SetDescription(self, Description: TCollection_HAsciiString) -> None: ...
	def SetElements(self, Elements: StepRepr_HArray1OfPropertyDefinitionRepresentation) -> None: ...
	def SetName(self, Name: TCollection_HAsciiString) -> None: ...

class StepRepr_FunctionallyDefinedTransformation(Standard_Transient):
	def __init__(self) -> None: ...
	def Description(self) -> TCollection_HAsciiString: ...
	def Init(self, aName: TCollection_HAsciiString, aDescription: TCollection_HAsciiString) -> None: ...
	def Name(self) -> TCollection_HAsciiString: ...
	def SetDescription(self, aDescription: TCollection_HAsciiString) -> None: ...
	def SetName(self, aName: TCollection_HAsciiString) -> None: ...

class StepRepr_ItemDefinedTransformation(Standard_Transient):
	def __init__(self) -> None: ...
	def Description(self) -> TCollection_HAsciiString: ...
	def Init(self, aName: TCollection_HAsciiString, aDescription: TCollection_HAsciiString, aTransformItem1: StepRepr_RepresentationItem, aTransformItem2: StepRepr_RepresentationItem) -> None: ...
	def Name(self) -> TCollection_HAsciiString: ...
	def SetDescription(self, aDescription: TCollection_HAsciiString) -> None: ...
	def SetName(self, aName: TCollection_HAsciiString) -> None: ...
	def SetTransformItem1(self, aItem: StepRepr_RepresentationItem) -> None: ...
	def SetTransformItem2(self, aItem: StepRepr_RepresentationItem) -> None: ...
	def TransformItem1(self) -> StepRepr_RepresentationItem: ...
	def TransformItem2(self) -> StepRepr_RepresentationItem: ...

class StepRepr_MaterialDesignation(Standard_Transient):
	def __init__(self) -> None: ...
	def Init(self, aName: TCollection_HAsciiString, aOfDefinition: StepRepr_CharacterizedDefinition) -> None: ...
	def Name(self) -> TCollection_HAsciiString: ...
	def OfDefinition(self) -> StepRepr_CharacterizedDefinition: ...
	def SetName(self, aName: TCollection_HAsciiString) -> None: ...
	def SetOfDefinition(self, aOfDefinition: StepRepr_CharacterizedDefinition) -> None: ...

class StepRepr_ProductConcept(Standard_Transient):
	def __init__(self) -> None: ...
	def Description(self) -> TCollection_HAsciiString: ...
	def HasDescription(self) -> bool: ...
	def Id(self) -> TCollection_HAsciiString: ...
	def Init(self, aId: TCollection_HAsciiString, aName: TCollection_HAsciiString, hasDescription: bool, aDescription: TCollection_HAsciiString, aMarketContext: StepBasic_ProductConceptContext) -> None: ...
	def MarketContext(self) -> StepBasic_ProductConceptContext: ...
	def Name(self) -> TCollection_HAsciiString: ...
	def SetDescription(self, Description: TCollection_HAsciiString) -> None: ...
	def SetId(self, Id: TCollection_HAsciiString) -> None: ...
	def SetMarketContext(self, MarketContext: StepBasic_ProductConceptContext) -> None: ...
	def SetName(self, Name: TCollection_HAsciiString) -> None: ...

class StepRepr_ProductDefinitionUsage(StepBasic_ProductDefinitionRelationship):
	def __init__(self) -> None: ...

class StepRepr_PropertyDefinition(Standard_Transient):
	def __init__(self) -> None: ...
	def Definition(self) -> StepRepr_CharacterizedDefinition: ...
	def Description(self) -> TCollection_HAsciiString: ...
	def HasDescription(self) -> bool: ...
	def Init(self, aName: TCollection_HAsciiString, hasDescription: bool, aDescription: TCollection_HAsciiString, aDefinition: StepRepr_CharacterizedDefinition) -> None: ...
	def Name(self) -> TCollection_HAsciiString: ...
	def SetDefinition(self, Definition: StepRepr_CharacterizedDefinition) -> None: ...
	def SetDescription(self, Description: TCollection_HAsciiString) -> None: ...
	def SetName(self, Name: TCollection_HAsciiString) -> None: ...

class StepRepr_PropertyDefinitionRelationship(Standard_Transient):
	def __init__(self) -> None: ...
	def Description(self) -> TCollection_HAsciiString: ...
	def Init(self, aName: TCollection_HAsciiString, aDescription: TCollection_HAsciiString, aRelatingPropertyDefinition: StepRepr_PropertyDefinition, aRelatedPropertyDefinition: StepRepr_PropertyDefinition) -> None: ...
	def Name(self) -> TCollection_HAsciiString: ...
	def RelatedPropertyDefinition(self) -> StepRepr_PropertyDefinition: ...
	def RelatingPropertyDefinition(self) -> StepRepr_PropertyDefinition: ...
	def SetDescription(self, Description: TCollection_HAsciiString) -> None: ...
	def SetName(self, Name: TCollection_HAsciiString) -> None: ...
	def SetRelatedPropertyDefinition(self, RelatedPropertyDefinition: StepRepr_PropertyDefinition) -> None: ...
	def SetRelatingPropertyDefinition(self, RelatingPropertyDefinition: StepRepr_PropertyDefinition) -> None: ...

class StepRepr_PropertyDefinitionRepresentation(Standard_Transient):
	def __init__(self) -> None: ...
	def Definition(self) -> StepRepr_RepresentedDefinition: ...
	def Init(self, aDefinition: StepRepr_RepresentedDefinition, aUsedRepresentation: StepRepr_Representation) -> None: ...
	def SetDefinition(self, Definition: StepRepr_RepresentedDefinition) -> None: ...
	def SetUsedRepresentation(self, UsedRepresentation: StepRepr_Representation) -> None: ...
	def UsedRepresentation(self) -> StepRepr_Representation: ...

class StepRepr_Representation(Standard_Transient):
	def __init__(self) -> None: ...
	def ContextOfItems(self) -> StepRepr_RepresentationContext: ...
	def Init(self, aName: TCollection_HAsciiString, aItems: StepRepr_HArray1OfRepresentationItem, aContextOfItems: StepRepr_RepresentationContext) -> None: ...
	def Items(self) -> StepRepr_HArray1OfRepresentationItem: ...
	def ItemsValue(self, num: int) -> StepRepr_RepresentationItem: ...
	def Name(self) -> TCollection_HAsciiString: ...
	def NbItems(self) -> int: ...
	def SetContextOfItems(self, aContextOfItems: StepRepr_RepresentationContext) -> None: ...
	def SetItems(self, aItems: StepRepr_HArray1OfRepresentationItem) -> None: ...
	def SetName(self, aName: TCollection_HAsciiString) -> None: ...

class StepRepr_RepresentationContext(Standard_Transient):
	def __init__(self) -> None: ...
	def ContextIdentifier(self) -> TCollection_HAsciiString: ...
	def ContextType(self) -> TCollection_HAsciiString: ...
	def Init(self, aContextIdentifier: TCollection_HAsciiString, aContextType: TCollection_HAsciiString) -> None: ...
	def SetContextIdentifier(self, aContextIdentifier: TCollection_HAsciiString) -> None: ...
	def SetContextType(self, aContextType: TCollection_HAsciiString) -> None: ...

class StepRepr_RepresentationItem(Standard_Transient):
	def __init__(self) -> None: ...
	def Init(self, aName: TCollection_HAsciiString) -> None: ...
	def Name(self) -> TCollection_HAsciiString: ...
	def SetName(self, aName: TCollection_HAsciiString) -> None: ...

class StepRepr_RepresentationMap(Standard_Transient):
	def __init__(self) -> None: ...
	def Init(self, aMappingOrigin: StepRepr_RepresentationItem, aMappedRepresentation: StepRepr_Representation) -> None: ...
	def MappedRepresentation(self) -> StepRepr_Representation: ...
	def MappingOrigin(self) -> StepRepr_RepresentationItem: ...
	def SetMappedRepresentation(self, aMappedRepresentation: StepRepr_Representation) -> None: ...
	def SetMappingOrigin(self, aMappingOrigin: StepRepr_RepresentationItem) -> None: ...

class StepRepr_RepresentationRelationship(Standard_Transient):
	def __init__(self) -> None: ...
	def Description(self) -> TCollection_HAsciiString: ...
	def Init(self, aName: TCollection_HAsciiString, aDescription: TCollection_HAsciiString, aRep1: StepRepr_Representation, aRep2: StepRepr_Representation) -> None: ...
	def Name(self) -> TCollection_HAsciiString: ...
	def Rep1(self) -> StepRepr_Representation: ...
	def Rep2(self) -> StepRepr_Representation: ...
	def SetDescription(self, aDescription: TCollection_HAsciiString) -> None: ...
	def SetName(self, aName: TCollection_HAsciiString) -> None: ...
	def SetRep1(self, aRep1: StepRepr_Representation) -> None: ...
	def SetRep2(self, aRep2: StepRepr_Representation) -> None: ...

class StepRepr_RepresentedDefinition(StepData_SelectType):
	def __init__(self) -> None: ...
	def CaseNum(self, ent: Standard_Transient) -> int: ...
	def GeneralProperty(self) -> StepBasic_GeneralProperty: ...
	def PropertyDefinition(self) -> StepRepr_PropertyDefinition: ...
	def PropertyDefinitionRelationship(self) -> StepRepr_PropertyDefinitionRelationship: ...
	def ShapeAspect(self) -> StepRepr_ShapeAspect: ...
	def ShapeAspectRelationship(self) -> StepRepr_ShapeAspectRelationship: ...

class StepRepr_ShapeAspect(Standard_Transient):
	def __init__(self) -> None: ...
	def Description(self) -> TCollection_HAsciiString: ...
	def Init(self, aName: TCollection_HAsciiString, aDescription: TCollection_HAsciiString, aOfShape: StepRepr_ProductDefinitionShape, aProductDefinitional: StepData_Logical) -> None: ...
	def Name(self) -> TCollection_HAsciiString: ...
	def OfShape(self) -> StepRepr_ProductDefinitionShape: ...
	def ProductDefinitional(self) -> StepData_Logical: ...
	def SetDescription(self, aDescription: TCollection_HAsciiString) -> None: ...
	def SetName(self, aName: TCollection_HAsciiString) -> None: ...
	def SetOfShape(self, aOfShape: StepRepr_ProductDefinitionShape) -> None: ...
	def SetProductDefinitional(self, aProductDefinitional: StepData_Logical) -> None: ...

class StepRepr_ShapeAspectRelationship(Standard_Transient):
	def __init__(self) -> None: ...
	def Description(self) -> TCollection_HAsciiString: ...
	def HasDescription(self) -> bool: ...
	def Init(self, aName: TCollection_HAsciiString, hasDescription: bool, aDescription: TCollection_HAsciiString, aRelatingShapeAspect: StepRepr_ShapeAspect, aRelatedShapeAspect: StepRepr_ShapeAspect) -> None: ...
	def Name(self) -> TCollection_HAsciiString: ...
	def RelatedShapeAspect(self) -> StepRepr_ShapeAspect: ...
	def RelatingShapeAspect(self) -> StepRepr_ShapeAspect: ...
	def SetDescription(self, Description: TCollection_HAsciiString) -> None: ...
	def SetName(self, Name: TCollection_HAsciiString) -> None: ...
	def SetRelatedShapeAspect(self, RelatedShapeAspect: StepRepr_ShapeAspect) -> None: ...
	def SetRelatingShapeAspect(self, RelatingShapeAspect: StepRepr_ShapeAspect) -> None: ...

class StepRepr_ShapeDefinition(StepData_SelectType):
	def __init__(self) -> None: ...
	def CaseNum(self, ent: Standard_Transient) -> int: ...
	def ProductDefinitionShape(self) -> StepRepr_ProductDefinitionShape: ...
	def ShapeAspect(self) -> StepRepr_ShapeAspect: ...
	def ShapeAspectRelationship(self) -> StepRepr_ShapeAspectRelationship: ...

class StepRepr_SuppliedPartRelationship(StepBasic_ProductDefinitionRelationship):
	def __init__(self) -> None: ...

class StepRepr_Transformation(StepData_SelectType):
	def __init__(self) -> None: ...
	def CaseNum(self, ent: Standard_Transient) -> int: ...
	def FunctionallyDefinedTransformation(self) -> StepRepr_FunctionallyDefinedTransformation: ...
	def ItemDefinedTransformation(self) -> StepRepr_ItemDefinedTransformation: ...

class StepRepr_AssemblyComponentUsage(StepRepr_ProductDefinitionUsage):
	def __init__(self) -> None: ...
	def HasReferenceDesignator(self) -> bool: ...
	@overload
	def Init(self, aProductDefinitionRelationship_Id: TCollection_HAsciiString, aProductDefinitionRelationship_Name: TCollection_HAsciiString, hasProductDefinitionRelationship_Description: bool, aProductDefinitionRelationship_Description: TCollection_HAsciiString, aProductDefinitionRelationship_RelatingProductDefinition: StepBasic_ProductDefinition, aProductDefinitionRelationship_RelatedProductDefinition: StepBasic_ProductDefinition, hasReferenceDesignator: bool, aReferenceDesignator: TCollection_HAsciiString) -> None: ...
	@overload
	def Init(self, aProductDefinitionRelationship_Id: TCollection_HAsciiString, aProductDefinitionRelationship_Name: TCollection_HAsciiString, hasProductDefinitionRelationship_Description: bool, aProductDefinitionRelationship_Description: TCollection_HAsciiString, aProductDefinitionRelationship_RelatingProductDefinition: StepBasic_ProductDefinitionOrReference, aProductDefinitionRelationship_RelatedProductDefinition: StepBasic_ProductDefinitionOrReference, hasReferenceDesignator: bool, aReferenceDesignator: TCollection_HAsciiString) -> None: ...
	def ReferenceDesignator(self) -> TCollection_HAsciiString: ...
	def SetReferenceDesignator(self, ReferenceDesignator: TCollection_HAsciiString) -> None: ...

class StepRepr_CharacterizedRepresentation(StepRepr_Representation):
	def __init__(self) -> None: ...
	def Description(self) -> TCollection_HAsciiString: ...
	def Init(self, theName: TCollection_HAsciiString, theDescription: TCollection_HAsciiString, theItems: StepRepr_HArray1OfRepresentationItem, theContextOfItems: StepRepr_RepresentationContext) -> None: ...
	def SetDescription(self, theDescription: TCollection_HAsciiString) -> None: ...

class StepRepr_CompShAspAndDatumFeatAndShAsp(StepRepr_ShapeAspect):
	def __init__(self) -> None: ...

class StepRepr_CompositeShapeAspect(StepRepr_ShapeAspect):
	def __init__(self) -> None: ...

class StepRepr_CompoundRepresentationItem(StepRepr_RepresentationItem):
	def __init__(self) -> None: ...
	def Init(self, aName: TCollection_HAsciiString, item_element: StepRepr_HArray1OfRepresentationItem) -> None: ...
	def ItemElement(self) -> StepRepr_HArray1OfRepresentationItem: ...
	def ItemElementValue(self, num: int) -> StepRepr_RepresentationItem: ...
	def NbItemElement(self) -> int: ...
	def SetItemElement(self, item_element: StepRepr_HArray1OfRepresentationItem) -> None: ...
	def SetItemElementValue(self, num: int, anelement: StepRepr_RepresentationItem) -> None: ...

class StepRepr_ConstructiveGeometryRepresentation(StepRepr_Representation):
	def __init__(self) -> None: ...

class StepRepr_ConstructiveGeometryRepresentationRelationship(StepRepr_RepresentationRelationship):
	def __init__(self) -> None: ...

class StepRepr_DefinitionalRepresentation(StepRepr_Representation):
	def __init__(self) -> None: ...

class StepRepr_DerivedShapeAspect(StepRepr_ShapeAspect):
	def __init__(self) -> None: ...

class StepRepr_DescriptiveRepresentationItem(StepRepr_RepresentationItem):
	def __init__(self) -> None: ...
	def Description(self) -> TCollection_HAsciiString: ...
	def Init(self, aName: TCollection_HAsciiString, aDescription: TCollection_HAsciiString) -> None: ...
	def SetDescription(self, aDescription: TCollection_HAsciiString) -> None: ...

class StepRepr_ExternallyDefinedRepresentation(StepRepr_Representation):
	def __init__(self) -> None: ...

class StepRepr_FeatureForDatumTargetRelationship(StepRepr_ShapeAspectRelationship):
	def __init__(self) -> None: ...

class StepRepr_GlobalUncertaintyAssignedContext(StepRepr_RepresentationContext):
	def __init__(self) -> None: ...
	def Init(self, aContextIdentifier: TCollection_HAsciiString, aContextType: TCollection_HAsciiString, aUncertainty: StepBasic_HArray1OfUncertaintyMeasureWithUnit) -> None: ...
	def NbUncertainty(self) -> int: ...
	def SetUncertainty(self, aUncertainty: StepBasic_HArray1OfUncertaintyMeasureWithUnit) -> None: ...
	def Uncertainty(self) -> StepBasic_HArray1OfUncertaintyMeasureWithUnit: ...
	def UncertaintyValue(self, num: int) -> StepBasic_UncertaintyMeasureWithUnit: ...

class StepRepr_GlobalUnitAssignedContext(StepRepr_RepresentationContext):
	def __init__(self) -> None: ...
	def Init(self, aContextIdentifier: TCollection_HAsciiString, aContextType: TCollection_HAsciiString, aUnits: StepBasic_HArray1OfNamedUnit) -> None: ...
	def NbUnits(self) -> int: ...
	def SetUnits(self, aUnits: StepBasic_HArray1OfNamedUnit) -> None: ...
	def Units(self) -> StepBasic_HArray1OfNamedUnit: ...
	def UnitsValue(self, num: int) -> StepBasic_NamedUnit: ...

class StepRepr_IntegerRepresentationItem(StepRepr_RepresentationItem):
	def __init__(self) -> None: ...
	def Init(self, theName: TCollection_HAsciiString, theValue: int) -> None: ...
	def SetValue(self, theValue: int) -> None: ...
	def Value(self) -> int: ...

class StepRepr_MakeFromUsageOption(StepRepr_ProductDefinitionUsage):
	def __init__(self) -> None: ...
	@overload
	def Init(self, aProductDefinitionRelationship_Id: TCollection_HAsciiString, aProductDefinitionRelationship_Name: TCollection_HAsciiString, hasProductDefinitionRelationship_Description: bool, aProductDefinitionRelationship_Description: TCollection_HAsciiString, aProductDefinitionRelationship_RelatingProductDefinition: StepBasic_ProductDefinition, aProductDefinitionRelationship_RelatedProductDefinition: StepBasic_ProductDefinition, aRanking: int, aRankingRationale: TCollection_HAsciiString, aQuantity: StepBasic_MeasureWithUnit) -> None: ...
	@overload
	def Init(self, aProductDefinitionRelationship_Id: TCollection_HAsciiString, aProductDefinitionRelationship_Name: TCollection_HAsciiString, hasProductDefinitionRelationship_Description: bool, aProductDefinitionRelationship_Description: TCollection_HAsciiString, aProductDefinitionRelationship_RelatingProductDefinition: StepBasic_ProductDefinitionOrReference, aProductDefinitionRelationship_RelatedProductDefinition: StepBasic_ProductDefinitionOrReference, aRanking: int, aRankingRationale: TCollection_HAsciiString, aQuantity: StepBasic_MeasureWithUnit) -> None: ...
	def Quantity(self) -> StepBasic_MeasureWithUnit: ...
	def Ranking(self) -> int: ...
	def RankingRationale(self) -> TCollection_HAsciiString: ...
	def SetQuantity(self, Quantity: StepBasic_MeasureWithUnit) -> None: ...
	def SetRanking(self, Ranking: int) -> None: ...
	def SetRankingRationale(self, RankingRationale: TCollection_HAsciiString) -> None: ...

class StepRepr_MappedItem(StepRepr_RepresentationItem):
	def __init__(self) -> None: ...
	def Init(self, aName: TCollection_HAsciiString, aMappingSource: StepRepr_RepresentationMap, aMappingTarget: StepRepr_RepresentationItem) -> None: ...
	def MappingSource(self) -> StepRepr_RepresentationMap: ...
	def MappingTarget(self) -> StepRepr_RepresentationItem: ...
	def SetMappingSource(self, aMappingSource: StepRepr_RepresentationMap) -> None: ...
	def SetMappingTarget(self, aMappingTarget: StepRepr_RepresentationItem) -> None: ...

class StepRepr_MaterialProperty(StepRepr_PropertyDefinition):
	def __init__(self) -> None: ...

class StepRepr_MaterialPropertyRepresentation(StepRepr_PropertyDefinitionRepresentation):
	def __init__(self) -> None: ...
	def DependentEnvironment(self) -> StepRepr_DataEnvironment: ...
	def Init(self, aPropertyDefinitionRepresentation_Definition: StepRepr_RepresentedDefinition, aPropertyDefinitionRepresentation_UsedRepresentation: StepRepr_Representation, aDependentEnvironment: StepRepr_DataEnvironment) -> None: ...
	def SetDependentEnvironment(self, DependentEnvironment: StepRepr_DataEnvironment) -> None: ...

class StepRepr_MeasureRepresentationItem(StepRepr_RepresentationItem):
	def __init__(self) -> None: ...
	def Init(self, aName: TCollection_HAsciiString, aValueComponent: StepBasic_MeasureValueMember, aUnitComponent: StepBasic_Unit) -> None: ...
	def Measure(self) -> StepBasic_MeasureWithUnit: ...
	def SetMeasure(self, Measure: StepBasic_MeasureWithUnit) -> None: ...

class StepRepr_ParametricRepresentationContext(StepRepr_RepresentationContext):
	def __init__(self) -> None: ...

class StepRepr_ProductDefinitionShape(StepRepr_PropertyDefinition):
	def __init__(self) -> None: ...

class StepRepr_ReprItemAndMeasureWithUnit(StepRepr_RepresentationItem):
	def __init__(self) -> None: ...
	def GetMeasureRepresentationItem(self) -> StepRepr_MeasureRepresentationItem: ...
	def GetMeasureWithUnit(self) -> StepBasic_MeasureWithUnit: ...
	def GetRepresentationItem(self) -> StepRepr_RepresentationItem: ...
	def Init(self, aMWU: StepBasic_MeasureWithUnit, aRI: StepRepr_RepresentationItem) -> None: ...
	def SetMeasureWithUnit(self, aMWU: StepBasic_MeasureWithUnit) -> None: ...

class StepRepr_ShapeAspectDerivingRelationship(StepRepr_ShapeAspectRelationship):
	def __init__(self) -> None: ...

class StepRepr_ShapeAspectTransition(StepRepr_ShapeAspectRelationship):
	def __init__(self) -> None: ...

class StepRepr_ShapeRepresentationRelationship(StepRepr_RepresentationRelationship):
	def __init__(self) -> None: ...

class StepRepr_StructuralResponseProperty(StepRepr_PropertyDefinition):
	def __init__(self) -> None: ...

class StepRepr_StructuralResponsePropertyDefinitionRepresentation(StepRepr_PropertyDefinitionRepresentation):
	def __init__(self) -> None: ...

class StepRepr_ValueRepresentationItem(StepRepr_RepresentationItem):
	def __init__(self) -> None: ...
	def Init(self, theName: TCollection_HAsciiString, theValueComponentMember: StepBasic_MeasureValueMember) -> None: ...
	def SetValueComponentMember(self, theValueComponentMember: StepBasic_MeasureValueMember) -> None: ...
	def ValueComponentMember(self) -> StepBasic_MeasureValueMember: ...

class StepRepr_Apex(StepRepr_DerivedShapeAspect):
	def __init__(self) -> None: ...

class StepRepr_CentreOfSymmetry(StepRepr_DerivedShapeAspect):
	def __init__(self) -> None: ...

class StepRepr_CompGroupShAspAndCompShAspAndDatumFeatAndShAsp(StepRepr_CompShAspAndDatumFeatAndShAsp):
	def __init__(self) -> None: ...

class StepRepr_CompositeGroupShapeAspect(StepRepr_CompositeShapeAspect):
	def __init__(self) -> None: ...

class StepRepr_ContinuosShapeAspect(StepRepr_CompositeShapeAspect):
	def __init__(self) -> None: ...

class StepRepr_Extension(StepRepr_DerivedShapeAspect):
	def __init__(self) -> None: ...

class StepRepr_GeometricAlignment(StepRepr_DerivedShapeAspect):
	def __init__(self) -> None: ...

class StepRepr_NextAssemblyUsageOccurrence(StepRepr_AssemblyComponentUsage):
	def __init__(self) -> None: ...

class StepRepr_ParallelOffset(StepRepr_DerivedShapeAspect):
	def __init__(self) -> None: ...
	def Init(self, theName: TCollection_HAsciiString, theDescription: TCollection_HAsciiString, theOfShape: StepRepr_ProductDefinitionShape, theProductDefinitional: StepData_Logical, theOffset: StepBasic_MeasureWithUnit) -> None: ...
	def Offset(self) -> StepBasic_MeasureWithUnit: ...
	def SetOffset(self, theOffset: StepBasic_MeasureWithUnit) -> None: ...

class StepRepr_PerpendicularTo(StepRepr_DerivedShapeAspect):
	def __init__(self) -> None: ...

class StepRepr_PromissoryUsageOccurrence(StepRepr_AssemblyComponentUsage):
	def __init__(self) -> None: ...

class StepRepr_QuantifiedAssemblyComponentUsage(StepRepr_AssemblyComponentUsage):
	def __init__(self) -> None: ...
	@overload
	def Init(self, aProductDefinitionRelationship_Id: TCollection_HAsciiString, aProductDefinitionRelationship_Name: TCollection_HAsciiString, hasProductDefinitionRelationship_Description: bool, aProductDefinitionRelationship_Description: TCollection_HAsciiString, aProductDefinitionRelationship_RelatingProductDefinition: StepBasic_ProductDefinition, aProductDefinitionRelationship_RelatedProductDefinition: StepBasic_ProductDefinition, hasAssemblyComponentUsage_ReferenceDesignator: bool, aAssemblyComponentUsage_ReferenceDesignator: TCollection_HAsciiString, aQuantity: StepBasic_MeasureWithUnit) -> None: ...
	@overload
	def Init(self, aProductDefinitionRelationship_Id: TCollection_HAsciiString, aProductDefinitionRelationship_Name: TCollection_HAsciiString, hasProductDefinitionRelationship_Description: bool, aProductDefinitionRelationship_Description: TCollection_HAsciiString, aProductDefinitionRelationship_RelatingProductDefinition: StepBasic_ProductDefinitionOrReference, aProductDefinitionRelationship_RelatedProductDefinition: StepBasic_ProductDefinitionOrReference, hasAssemblyComponentUsage_ReferenceDesignator: bool, aAssemblyComponentUsage_ReferenceDesignator: TCollection_HAsciiString, aQuantity: StepBasic_MeasureWithUnit) -> None: ...
	def Quantity(self) -> StepBasic_MeasureWithUnit: ...
	def SetQuantity(self, Quantity: StepBasic_MeasureWithUnit) -> None: ...

class StepRepr_ReprItemAndLengthMeasureWithUnit(StepRepr_ReprItemAndMeasureWithUnit):
	def __init__(self) -> None: ...
	def GetLengthMeasureWithUnit(self) -> StepBasic_LengthMeasureWithUnit: ...
	def SetLengthMeasureWithUnit(self, aLMWU: StepBasic_LengthMeasureWithUnit) -> None: ...

class StepRepr_ReprItemAndPlaneAngleMeasureWithUnit(StepRepr_ReprItemAndMeasureWithUnit):
	def __init__(self) -> None: ...
	def GetPlaneAngleMeasureWithUnit(self) -> StepBasic_PlaneAngleMeasureWithUnit: ...
	def SetPlaneAngleMeasureWithUnit(self, aLMWU: StepBasic_PlaneAngleMeasureWithUnit) -> None: ...

class StepRepr_RepresentationRelationshipWithTransformation(StepRepr_ShapeRepresentationRelationship):
	def __init__(self) -> None: ...
	def Init(self, aName: TCollection_HAsciiString, aDescription: TCollection_HAsciiString, aRep1: StepRepr_Representation, aRep2: StepRepr_Representation, aTransf: StepRepr_Transformation) -> None: ...
	def SetTransformationOperator(self, aTrans: StepRepr_Transformation) -> None: ...
	def TransformationOperator(self) -> StepRepr_Transformation: ...

class StepRepr_SpecifiedHigherUsageOccurrence(StepRepr_AssemblyComponentUsage):
	def __init__(self) -> None: ...
	@overload
	def Init(self, aProductDefinitionRelationship_Id: TCollection_HAsciiString, aProductDefinitionRelationship_Name: TCollection_HAsciiString, hasProductDefinitionRelationship_Description: bool, aProductDefinitionRelationship_Description: TCollection_HAsciiString, aProductDefinitionRelationship_RelatingProductDefinition: StepBasic_ProductDefinition, aProductDefinitionRelationship_RelatedProductDefinition: StepBasic_ProductDefinition, hasAssemblyComponentUsage_ReferenceDesignator: bool, aAssemblyComponentUsage_ReferenceDesignator: TCollection_HAsciiString, aUpperUsage: StepRepr_AssemblyComponentUsage, aNextUsage: StepRepr_NextAssemblyUsageOccurrence) -> None: ...
	@overload
	def Init(self, aProductDefinitionRelationship_Id: TCollection_HAsciiString, aProductDefinitionRelationship_Name: TCollection_HAsciiString, hasProductDefinitionRelationship_Description: bool, aProductDefinitionRelationship_Description: TCollection_HAsciiString, aProductDefinitionRelationship_RelatingProductDefinition: StepBasic_ProductDefinitionOrReference, aProductDefinitionRelationship_RelatedProductDefinition: StepBasic_ProductDefinitionOrReference, hasAssemblyComponentUsage_ReferenceDesignator: bool, aAssemblyComponentUsage_ReferenceDesignator: TCollection_HAsciiString, aUpperUsage: StepRepr_AssemblyComponentUsage, aNextUsage: StepRepr_NextAssemblyUsageOccurrence) -> None: ...
	def NextUsage(self) -> StepRepr_NextAssemblyUsageOccurrence: ...
	def SetNextUsage(self, NextUsage: StepRepr_NextAssemblyUsageOccurrence) -> None: ...
	def SetUpperUsage(self, UpperUsage: StepRepr_AssemblyComponentUsage) -> None: ...
	def UpperUsage(self) -> StepRepr_AssemblyComponentUsage: ...

class StepRepr_Tangent(StepRepr_DerivedShapeAspect):
	def __init__(self) -> None: ...

class StepRepr_ValueRange(StepRepr_CompoundRepresentationItem):
	def __init__(self) -> None: ...

class StepRepr_AllAroundShapeAspect(StepRepr_ContinuosShapeAspect):
	def __init__(self) -> None: ...

class StepRepr_BetweenShapeAspect(StepRepr_ContinuosShapeAspect):
	def __init__(self) -> None: ...

class StepRepr_ReprItemAndLengthMeasureWithUnitAndQRI(StepRepr_ReprItemAndMeasureWithUnitAndQRI):
	def __init__(self) -> None: ...
	def GetLengthMeasureWithUnit(self) -> StepBasic_LengthMeasureWithUnit: ...
	def SetLengthMeasureWithUnit(self, aLMWU: StepBasic_LengthMeasureWithUnit) -> None: ...

class StepRepr_ReprItemAndPlaneAngleMeasureWithUnitAndQRI(StepRepr_ReprItemAndMeasureWithUnitAndQRI):
	def __init__(self) -> None: ...
	def GetPlaneAngleMeasureWithUnit(self) -> StepBasic_PlaneAngleMeasureWithUnit: ...
	def SetPlaneAngleMeasureWithUnit(self, aLMWU: StepBasic_PlaneAngleMeasureWithUnit) -> None: ...

class StepRepr_ShapeRepresentationRelationshipWithTransformation(StepRepr_RepresentationRelationshipWithTransformation):
	def __init__(self) -> None: ...

#classnotwrapped
class StepRepr_ReprItemAndMeasureWithUnitAndQRI: ...

# harray1 classes

class StepRepr_HArray1OfMaterialPropertyRepresentation(StepRepr_Array1OfMaterialPropertyRepresentation, Standard_Transient):
    def __init__(self, theLower: int, theUpper: int) -> None: ...
    def Array1(self) -> StepRepr_Array1OfMaterialPropertyRepresentation: ...


class StepRepr_HArray1OfRepresentationItem(StepRepr_Array1OfRepresentationItem, Standard_Transient):
    def __init__(self, theLower: int, theUpper: int) -> None: ...
    def Array1(self) -> StepRepr_Array1OfRepresentationItem: ...


class StepRepr_HArray1OfPropertyDefinitionRepresentation(StepRepr_Array1OfPropertyDefinitionRepresentation, Standard_Transient):
    def __init__(self, theLower: int, theUpper: int) -> None: ...
    def Array1(self) -> StepRepr_Array1OfPropertyDefinitionRepresentation: ...


class StepRepr_HArray1OfShapeAspect(StepRepr_Array1OfShapeAspect, Standard_Transient):
    def __init__(self, theLower: int, theUpper: int) -> None: ...
    def Array1(self) -> StepRepr_Array1OfShapeAspect: ...

# harray2 classes
# hsequence classes

class StepRepr_HSequenceOfRepresentationItem(StepRepr_SequenceOfRepresentationItem, Standard_Transient):
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, other: StepRepr_SequenceOfRepresentationItem) -> None: ...
    def Sequence(self) -> StepRepr_SequenceOfRepresentationItem: ...
    def Append(self, theSequence: StepRepr_SequenceOfRepresentationItem) -> None: ...


class StepRepr_HSequenceOfMaterialPropertyRepresentation(StepRepr_SequenceOfMaterialPropertyRepresentation, Standard_Transient):
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, other: StepRepr_SequenceOfMaterialPropertyRepresentation) -> None: ...
    def Sequence(self) -> StepRepr_SequenceOfMaterialPropertyRepresentation: ...
    def Append(self, theSequence: StepRepr_SequenceOfMaterialPropertyRepresentation) -> None: ...





from enum import IntEnum
from typing import overload, NewType, Optional, Tuple

from OCC.Core.Standard import *
from OCC.Core.NCollection import *
from OCC.Core.gp import *
from OCC.Core.Adaptor3d import *
from OCC.Core.GeomAbs import *

#the following typedef cannot be wrapped as is
IntSurf_Allocator = NewType('IntSurf_Allocator', Any)

class IntSurf_ListOfPntOn2S:
    def __init__(self) -> None: ...
    def __len__(self) -> int: ...
    def Size(self) -> int: ...
    def Clear(self) -> None: ...
    def First(self) -> IntSurf_PntOn2S: ...
    def Last(self) -> IntSurf_PntOn2S: ...
    def Append(self, theItem: IntSurf_PntOn2S) -> IntSurf_PntOn2S: ...
    def Prepend(self, theItem: IntSurf_PntOn2S) -> IntSurf_PntOn2S: ...
    def RemoveFirst(self) -> None: ...
    def Reverse(self) -> None: ...
    def Value(self, theIndex: int) -> IntSurf_PntOn2S: ...
    def SetValue(self, theIndex: int, theValue: IntSurf_PntOn2S) -> None: ...

class IntSurf_SequenceOfCouple:
    def __init__(self) -> None: ...
    def __len__(self) -> int: ...
    def Size(self) -> int: ...
    def Clear(self) -> None: ...
    def First(self) -> IntSurf_Couple: ...
    def Last(self) -> IntSurf_Couple: ...
    def Length(self) -> int: ...
    def Append(self, theItem: IntSurf_Couple) -> IntSurf_Couple: ...
    def Prepend(self, theItem: IntSurf_Couple) -> IntSurf_Couple: ...
    def RemoveFirst(self) -> None: ...
    def Reverse(self) -> None: ...
    def Value(self, theIndex: int) -> IntSurf_Couple: ...
    def SetValue(self, theIndex: int, theValue: IntSurf_Couple) -> None: ...

class IntSurf_SequenceOfInteriorPoint:
    def __init__(self) -> None: ...
    def __len__(self) -> int: ...
    def Size(self) -> int: ...
    def Clear(self) -> None: ...
    def First(self) -> IntSurf_InteriorPoint: ...
    def Last(self) -> IntSurf_InteriorPoint: ...
    def Length(self) -> int: ...
    def Append(self, theItem: IntSurf_InteriorPoint) -> IntSurf_InteriorPoint: ...
    def Prepend(self, theItem: IntSurf_InteriorPoint) -> IntSurf_InteriorPoint: ...
    def RemoveFirst(self) -> None: ...
    def Reverse(self) -> None: ...
    def Value(self, theIndex: int) -> IntSurf_InteriorPoint: ...
    def SetValue(self, theIndex: int, theValue: IntSurf_InteriorPoint) -> None: ...

class IntSurf_SequenceOfPathPoint:
    def __init__(self) -> None: ...
    def __len__(self) -> int: ...
    def Size(self) -> int: ...
    def Clear(self) -> None: ...
    def First(self) -> IntSurf_PathPoint: ...
    def Last(self) -> IntSurf_PathPoint: ...
    def Length(self) -> int: ...
    def Append(self, theItem: IntSurf_PathPoint) -> IntSurf_PathPoint: ...
    def Prepend(self, theItem: IntSurf_PathPoint) -> IntSurf_PathPoint: ...
    def RemoveFirst(self) -> None: ...
    def Reverse(self) -> None: ...
    def Value(self, theIndex: int) -> IntSurf_PathPoint: ...
    def SetValue(self, theIndex: int, theValue: IntSurf_PathPoint) -> None: ...

class IntSurf_SequenceOfPntOn2S:
    def __init__(self) -> None: ...
    def __len__(self) -> int: ...
    def Size(self) -> int: ...
    def Clear(self) -> None: ...
    def First(self) -> IntSurf_PntOn2S: ...
    def Last(self) -> IntSurf_PntOn2S: ...
    def Length(self) -> int: ...
    def Append(self, theItem: IntSurf_PntOn2S) -> IntSurf_PntOn2S: ...
    def Prepend(self, theItem: IntSurf_PntOn2S) -> IntSurf_PntOn2S: ...
    def RemoveFirst(self) -> None: ...
    def Reverse(self) -> None: ...
    def Value(self, theIndex: int) -> IntSurf_PntOn2S: ...
    def SetValue(self, theIndex: int, theValue: IntSurf_PntOn2S) -> None: ...

class IntSurf_Situation(IntEnum):
	IntSurf_Inside: int = ...
	IntSurf_Outside: int = ...
	IntSurf_Unknown: int = ...
IntSurf_Inside = IntSurf_Situation.IntSurf_Inside
IntSurf_Outside = IntSurf_Situation.IntSurf_Outside
IntSurf_Unknown = IntSurf_Situation.IntSurf_Unknown

class IntSurf_TypeTrans(IntEnum):
	IntSurf_In: int = ...
	IntSurf_Out: int = ...
	IntSurf_Touch: int = ...
	IntSurf_Undecided: int = ...
IntSurf_In = IntSurf_TypeTrans.IntSurf_In
IntSurf_Out = IntSurf_TypeTrans.IntSurf_Out
IntSurf_Touch = IntSurf_TypeTrans.IntSurf_Touch
IntSurf_Undecided = IntSurf_TypeTrans.IntSurf_Undecided

class intsurf:
	@staticmethod
	def MakeTransition(TgFirst: gp_Vec, TgSecond: gp_Vec, Normal: gp_Dir, TFirst: IntSurf_Transition, TSecond: IntSurf_Transition) -> None: ...
	@staticmethod
	def SetPeriod(theFirstSurf: Adaptor3d_HSurface, theSecondSurf: Adaptor3d_HSurface, theArrOfPeriod_list: List[float]) -> None: ...

class IntSurf_Couple:
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, Index1: int, Index2: int) -> None: ...
	def First(self) -> int: ...
	def Second(self) -> int: ...

class IntSurf_InteriorPoint:
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, P: gp_Pnt, U: float, V: float, Direc: gp_Vec, Direc2d: gp_Vec2d) -> None: ...
	def Direction(self) -> gp_Vec: ...
	def Direction2d(self) -> gp_Vec2d: ...
	def Parameters(self) -> Tuple[float, float]: ...
	def SetValue(self, P: gp_Pnt, U: float, V: float, Direc: gp_Vec, Direc2d: gp_Vec2d) -> None: ...
	def UParameter(self) -> float: ...
	def VParameter(self) -> float: ...
	def Value(self) -> gp_Pnt: ...

class IntSurf_InteriorPointTool:
	@staticmethod
	def Direction2d(PStart: IntSurf_InteriorPoint) -> gp_Dir2d: ...
	@staticmethod
	def Direction3d(PStart: IntSurf_InteriorPoint) -> gp_Vec: ...
	@staticmethod
	def Value2d(PStart: IntSurf_InteriorPoint) -> Tuple[float, float]: ...
	@staticmethod
	def Value3d(PStart: IntSurf_InteriorPoint) -> gp_Pnt: ...

class IntSurf_LineOn2S(Standard_Transient):
	def __init__(self, theAllocator: Optional[IntSurf_Allocator] = 0) -> None: ...
	def Add(self, P: IntSurf_PntOn2S) -> None: ...
	def Clear(self) -> None: ...
	def InsertBefore(self, I: int, P: IntSurf_PntOn2S) -> None: ...
	def IsOutBox(self, theP: gp_Pnt) -> bool: ...
	def IsOutSurf1Box(self, theP: gp_Pnt2d) -> bool: ...
	def IsOutSurf2Box(self, theP: gp_Pnt2d) -> bool: ...
	def NbPoints(self) -> int: ...
	def RemovePoint(self, I: int) -> None: ...
	def Reverse(self) -> None: ...
	def SetUV(self, Index: int, OnFirst: bool, U: float, V: float) -> None: ...
	def Split(self, Index: int) -> IntSurf_LineOn2S: ...
	@overload
	def Value(self, Index: int) -> IntSurf_PntOn2S: ...
	@overload
	def Value(self, Index: int, P: IntSurf_PntOn2S) -> None: ...

class IntSurf_PathPoint:
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, P: gp_Pnt, U: float, V: float) -> None: ...
	def AddUV(self, U: float, V: float) -> None: ...
	def Direction2d(self) -> gp_Dir2d: ...
	def Direction3d(self) -> gp_Vec: ...
	def IsPassingPnt(self) -> bool: ...
	def IsTangent(self) -> bool: ...
	def Multiplicity(self) -> int: ...
	def Parameters(self, Index: int) -> Tuple[float, float]: ...
	def SetDirections(self, V: gp_Vec, D: gp_Dir2d) -> None: ...
	def SetPassing(self, Pass: bool) -> None: ...
	def SetTangency(self, Tang: bool) -> None: ...
	def SetValue(self, P: gp_Pnt, U: float, V: float) -> None: ...
	def Value(self) -> gp_Pnt: ...
	def Value2d(self) -> Tuple[float, float]: ...

class IntSurf_PathPointTool:
	@staticmethod
	def Direction2d(PStart: IntSurf_PathPoint) -> gp_Dir2d: ...
	@staticmethod
	def Direction3d(PStart: IntSurf_PathPoint) -> gp_Vec: ...
	@staticmethod
	def IsPassingPnt(PStart: IntSurf_PathPoint) -> bool: ...
	@staticmethod
	def IsTangent(PStart: IntSurf_PathPoint) -> bool: ...
	@staticmethod
	def Multiplicity(PStart: IntSurf_PathPoint) -> int: ...
	@staticmethod
	def Parameters(PStart: IntSurf_PathPoint, Mult: int) -> Tuple[float, float]: ...
	@staticmethod
	def Value2d(PStart: IntSurf_PathPoint) -> Tuple[float, float]: ...
	@staticmethod
	def Value3d(PStart: IntSurf_PathPoint) -> gp_Pnt: ...

class IntSurf_PntOn2S:
	def __init__(self) -> None: ...
	def IsSame(self, theOtherPoint: IntSurf_PntOn2S, theTol3D: Optional[float] = 0.0, theTol2D: Optional[float] = -1.0) -> bool: ...
	def Parameters(self) -> Tuple[float, float, float, float]: ...
	def ParametersOnS1(self) -> Tuple[float, float]: ...
	def ParametersOnS2(self) -> Tuple[float, float]: ...
	def ParametersOnSurface(self, OnFirst: bool) -> Tuple[float, float]: ...
	@overload
	def SetValue(self, Pt: gp_Pnt) -> None: ...
	@overload
	def SetValue(self, Pt: gp_Pnt, OnFirst: bool, U: float, V: float) -> None: ...
	@overload
	def SetValue(self, Pt: gp_Pnt, U1: float, V1: float, U2: float, V2: float) -> None: ...
	@overload
	def SetValue(self, OnFirst: bool, U: float, V: float) -> None: ...
	@overload
	def SetValue(self, U1: float, V1: float, U2: float, V2: float) -> None: ...
	def Value(self) -> gp_Pnt: ...
	def ValueOnSurface(self, OnFirst: bool) -> gp_Pnt2d: ...

class IntSurf_Quadric:
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, P: gp_Pln) -> None: ...
	@overload
	def __init__(self, C: gp_Cylinder) -> None: ...
	@overload
	def __init__(self, S: gp_Sphere) -> None: ...
	@overload
	def __init__(self, C: gp_Cone) -> None: ...
	@overload
	def __init__(self, T: gp_Torus) -> None: ...
	def Cone(self) -> gp_Cone: ...
	def Cylinder(self) -> gp_Cylinder: ...
	def D1(self, U: float, V: float, P: gp_Pnt, D1U: gp_Vec, D1V: gp_Vec) -> None: ...
	def DN(self, U: float, V: float, Nu: int, Nv: int) -> gp_Vec: ...
	def Distance(self, P: gp_Pnt) -> float: ...
	def Gradient(self, P: gp_Pnt) -> gp_Vec: ...
	@overload
	def Normale(self, U: float, V: float) -> gp_Vec: ...
	@overload
	def Normale(self, P: gp_Pnt) -> gp_Vec: ...
	def Parameters(self, P: gp_Pnt) -> Tuple[float, float]: ...
	def Plane(self) -> gp_Pln: ...
	@overload
	def SetValue(self, P: gp_Pln) -> None: ...
	@overload
	def SetValue(self, C: gp_Cylinder) -> None: ...
	@overload
	def SetValue(self, S: gp_Sphere) -> None: ...
	@overload
	def SetValue(self, C: gp_Cone) -> None: ...
	@overload
	def SetValue(self, T: gp_Torus) -> None: ...
	def Sphere(self) -> gp_Sphere: ...
	def Torus(self) -> gp_Torus: ...
	def TypeQuadric(self) -> GeomAbs_SurfaceType: ...
	def ValAndGrad(self, P: gp_Pnt, Grad: gp_Vec) -> float: ...
	def Value(self, U: float, V: float) -> gp_Pnt: ...

class IntSurf_QuadricTool:
	@staticmethod
	def Gradient(Quad: IntSurf_Quadric, X: float, Y: float, Z: float, V: gp_Vec) -> None: ...
	@staticmethod
	def Tolerance(Quad: IntSurf_Quadric) -> float: ...
	@staticmethod
	def Value(Quad: IntSurf_Quadric, X: float, Y: float, Z: float) -> float: ...
	@staticmethod
	def ValueAndGradient(Quad: IntSurf_Quadric, X: float, Y: float, Z: float, Grad: gp_Vec) -> float: ...

class IntSurf_Transition:
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, Tangent: bool, Type: IntSurf_TypeTrans) -> None: ...
	@overload
	def __init__(self, Tangent: bool, Situ: IntSurf_Situation, Oppos: bool) -> None: ...
	def IsOpposite(self) -> bool: ...
	def IsTangent(self) -> bool: ...
	@overload
	def SetValue(self, Tangent: bool, Type: IntSurf_TypeTrans) -> None: ...
	@overload
	def SetValue(self, Tangent: bool, Situ: IntSurf_Situation, Oppos: bool) -> None: ...
	@overload
	def SetValue(self) -> None: ...
	def Situation(self) -> IntSurf_Situation: ...
	def TransitionType(self) -> IntSurf_TypeTrans: ...

# harray1 classes
# harray2 classes
# hsequence classes

intsurf_MakeTransition = intsurf.MakeTransition
intsurf_SetPeriod = intsurf.SetPeriod
IntSurf_InteriorPointTool_Direction2d = IntSurf_InteriorPointTool.Direction2d
IntSurf_InteriorPointTool_Direction3d = IntSurf_InteriorPointTool.Direction3d
IntSurf_InteriorPointTool_Value2d = IntSurf_InteriorPointTool.Value2d
IntSurf_InteriorPointTool_Value3d = IntSurf_InteriorPointTool.Value3d
IntSurf_PathPointTool_Direction2d = IntSurf_PathPointTool.Direction2d
IntSurf_PathPointTool_Direction3d = IntSurf_PathPointTool.Direction3d
IntSurf_PathPointTool_IsPassingPnt = IntSurf_PathPointTool.IsPassingPnt
IntSurf_PathPointTool_IsTangent = IntSurf_PathPointTool.IsTangent
IntSurf_PathPointTool_Multiplicity = IntSurf_PathPointTool.Multiplicity
IntSurf_PathPointTool_Parameters = IntSurf_PathPointTool.Parameters
IntSurf_PathPointTool_Value2d = IntSurf_PathPointTool.Value2d
IntSurf_PathPointTool_Value3d = IntSurf_PathPointTool.Value3d
IntSurf_QuadricTool_Gradient = IntSurf_QuadricTool.Gradient
IntSurf_QuadricTool_Tolerance = IntSurf_QuadricTool.Tolerance
IntSurf_QuadricTool_Value = IntSurf_QuadricTool.Value
IntSurf_QuadricTool_ValueAndGradient = IntSurf_QuadricTool.ValueAndGradient



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

"""
TopCnx module, see official documentation at
https://www.opencascade.com/doc/occt-7.4.0/refman/html/package_topcnx.html
"""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _TopCnx
else:
    import _TopCnx

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


class SwigPyIterator(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _TopCnx.delete_SwigPyIterator

    def value(self) -> "PyObject *":
        return _TopCnx.SwigPyIterator_value(self)

    def incr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _TopCnx.SwigPyIterator_incr(self, n)

    def decr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _TopCnx.SwigPyIterator_decr(self, n)

    def distance(self, x: "SwigPyIterator") -> "ptrdiff_t":
        return _TopCnx.SwigPyIterator_distance(self, x)

    def equal(self, x: "SwigPyIterator") -> "bool":
        return _TopCnx.SwigPyIterator_equal(self, x)

    def copy(self) -> "swig::SwigPyIterator *":
        return _TopCnx.SwigPyIterator_copy(self)

    def next(self) -> "PyObject *":
        return _TopCnx.SwigPyIterator_next(self)

    def __next__(self) -> "PyObject *":
        return _TopCnx.SwigPyIterator___next__(self)

    def previous(self) -> "PyObject *":
        return _TopCnx.SwigPyIterator_previous(self)

    def advance(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _TopCnx.SwigPyIterator_advance(self, n)

    def __eq__(self, x: "SwigPyIterator") -> "bool":
        return _TopCnx.SwigPyIterator___eq__(self, x)

    def __ne__(self, x: "SwigPyIterator") -> "bool":
        return _TopCnx.SwigPyIterator___ne__(self, x)

    def __iadd__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _TopCnx.SwigPyIterator___iadd__(self, n)

    def __isub__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _TopCnx.SwigPyIterator___isub__(self, n)

    def __add__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _TopCnx.SwigPyIterator___add__(self, n)

    def __sub__(self, *args) -> "ptrdiff_t":
        return _TopCnx.SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self

# Register SwigPyIterator in _TopCnx:
_TopCnx.SwigPyIterator_swigregister(SwigPyIterator)


def _dumps_object(klass):
    """ Overwrite default string output for any wrapped object.
    By default, __repr__ method returns something like:
    <OCC.Core.TopoDS.TopoDS_Shape; proxy of <Swig Object of type 'TopoDS_Shape *' at 0x02BB0758> >
    This is too much verbose.
    We prefer :
    <class 'gp_Pnt'>
    or
    <class 'TopoDS_Shape'>
    """
    klass_name = str(klass.__class__).split(".")[3].split("'")[0]
    repr_string = "<class '" + klass_name + "'"
# for TopoDS_Shape, we also look for the base type
    if klass_name == "TopoDS_Shape":
        if klass.IsNull():
            repr_string += ": Null>"
            return repr_string
        st = klass.ShapeType()
        types = {OCC.Core.TopAbs.TopAbs_VERTEX: "Vertex",
                 OCC.Core.TopAbs.TopAbs_SOLID: "Solid",
                 OCC.Core.TopAbs.TopAbs_EDGE: "Edge",
                 OCC.Core.TopAbs.TopAbs_FACE: "Face",
                 OCC.Core.TopAbs.TopAbs_SHELL: "Shell",
                 OCC.Core.TopAbs.TopAbs_WIRE: "Wire",
                 OCC.Core.TopAbs.TopAbs_COMPOUND: "Compound",
                 OCC.Core.TopAbs.TopAbs_COMPSOLID: "Compsolid"}
        repr_string += "; Type:%s" % types[st]        
    elif hasattr(klass, "IsNull"):
        if klass.IsNull():
            repr_string += "; Null"
    repr_string += ">"
    return repr_string


def process_exception(error: "Standard_Failure", method_name: "std::string", class_name: "std::string") -> "void":
    return _TopCnx.process_exception(error, method_name, class_name)

from six import with_metaclass
import warnings
from OCC.Wrapper.wrapper_utils import Proxy, deprecated

import OCC.Core.Standard
import OCC.Core.NCollection
import OCC.Core.gp
import OCC.Core.TColStd
import OCC.Core.TCollection
import OCC.Core.TopAbs

from enum import IntEnum
from OCC.Core.Exception import *



class TopCnx_EdgeFaceTransition(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Creates an empty algorithm.

        Returns
        -------
        None

        """
        _TopCnx.TopCnx_EdgeFaceTransition_swiginit(self, _TopCnx.new_TopCnx_EdgeFaceTransition(*args))

    def AddInterference(self, *args) -> "void":
        r"""
        Add a curve element to the boundary. or is the orientation of the interference on the boundary curve. tr is the transition of the interference. btr is the boundary transition of the interference.

        Parameters
        ----------
        Tole: float
        Tang: gp_Dir
        Norm: gp_Dir
        Curv: float
        Or: TopAbs_Orientation
        Tr: TopAbs_Orientation
        BTr: TopAbs_Orientation

        Returns
        -------
        None

        """
        return _TopCnx.TopCnx_EdgeFaceTransition_AddInterference(self, *args)

    def BoundaryTransition(self, *args) -> "TopAbs_Orientation":
        r"""
        Returns the current cumulated boundarytransition.

        Returns
        -------
        TopAbs_Orientation

        """
        return _TopCnx.TopCnx_EdgeFaceTransition_BoundaryTransition(self, *args)

    def Reset(self, *args) -> "void":
        r"""
        Initialize the algorithm with the local description of the edge.

        Parameters
        ----------
        Tgt: gp_Dir
        Norm: gp_Dir
        Curv: float

        Returns
        -------
        None
        Initialize the algorithm with a linear edge.

        Parameters
        ----------
        Tgt: gp_Dir

        Returns
        -------
        None

        """
        return _TopCnx.TopCnx_EdgeFaceTransition_Reset(self, *args)

    def Transition(self, *args) -> "TopAbs_Orientation":
        r"""
        Returns the current cumulated transition.

        Returns
        -------
        TopAbs_Orientation

        """
        return _TopCnx.TopCnx_EdgeFaceTransition_Transition(self, *args)

    __repr__ = _dumps_object

    __swig_destroy__ = _TopCnx.delete_TopCnx_EdgeFaceTransition

# Register TopCnx_EdgeFaceTransition in _TopCnx:
_TopCnx.TopCnx_EdgeFaceTransition_swigregister(TopCnx_EdgeFaceTransition)








# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

"""
AppCont module, see official documentation at
https://www.opencascade.com/doc/occt-7.4.0/refman/html/package_appcont.html
"""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _AppCont
else:
    import _AppCont

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


class SwigPyIterator(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _AppCont.delete_SwigPyIterator

    def value(self) -> "PyObject *":
        return _AppCont.SwigPyIterator_value(self)

    def incr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _AppCont.SwigPyIterator_incr(self, n)

    def decr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _AppCont.SwigPyIterator_decr(self, n)

    def distance(self, x: "SwigPyIterator") -> "ptrdiff_t":
        return _AppCont.SwigPyIterator_distance(self, x)

    def equal(self, x: "SwigPyIterator") -> "bool":
        return _AppCont.SwigPyIterator_equal(self, x)

    def copy(self) -> "swig::SwigPyIterator *":
        return _AppCont.SwigPyIterator_copy(self)

    def next(self) -> "PyObject *":
        return _AppCont.SwigPyIterator_next(self)

    def __next__(self) -> "PyObject *":
        return _AppCont.SwigPyIterator___next__(self)

    def previous(self) -> "PyObject *":
        return _AppCont.SwigPyIterator_previous(self)

    def advance(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _AppCont.SwigPyIterator_advance(self, n)

    def __eq__(self, x: "SwigPyIterator") -> "bool":
        return _AppCont.SwigPyIterator___eq__(self, x)

    def __ne__(self, x: "SwigPyIterator") -> "bool":
        return _AppCont.SwigPyIterator___ne__(self, x)

    def __iadd__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _AppCont.SwigPyIterator___iadd__(self, n)

    def __isub__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _AppCont.SwigPyIterator___isub__(self, n)

    def __add__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _AppCont.SwigPyIterator___add__(self, n)

    def __sub__(self, *args) -> "ptrdiff_t":
        return _AppCont.SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self

# Register SwigPyIterator in _AppCont:
_AppCont.SwigPyIterator_swigregister(SwigPyIterator)


def _dumps_object(klass):
    """ Overwrite default string output for any wrapped object.
    By default, __repr__ method returns something like:
    <OCC.Core.TopoDS.TopoDS_Shape; proxy of <Swig Object of type 'TopoDS_Shape *' at 0x02BB0758> >
    This is too much verbose.
    We prefer :
    <class 'gp_Pnt'>
    or
    <class 'TopoDS_Shape'>
    """
    klass_name = str(klass.__class__).split(".")[3].split("'")[0]
    repr_string = "<class '" + klass_name + "'"
# for TopoDS_Shape, we also look for the base type
    if klass_name == "TopoDS_Shape":
        if klass.IsNull():
            repr_string += ": Null>"
            return repr_string
        st = klass.ShapeType()
        types = {OCC.Core.TopAbs.TopAbs_VERTEX: "Vertex",
                 OCC.Core.TopAbs.TopAbs_SOLID: "Solid",
                 OCC.Core.TopAbs.TopAbs_EDGE: "Edge",
                 OCC.Core.TopAbs.TopAbs_FACE: "Face",
                 OCC.Core.TopAbs.TopAbs_SHELL: "Shell",
                 OCC.Core.TopAbs.TopAbs_WIRE: "Wire",
                 OCC.Core.TopAbs.TopAbs_COMPOUND: "Compound",
                 OCC.Core.TopAbs.TopAbs_COMPSOLID: "Compsolid"}
        repr_string += "; Type:%s" % types[st]        
    elif hasattr(klass, "IsNull"):
        if klass.IsNull():
            repr_string += "; Null"
    repr_string += ">"
    return repr_string


def process_exception(error: "Standard_Failure", method_name: "std::string", class_name: "std::string") -> "void":
    return _AppCont.process_exception(error, method_name, class_name)

from six import with_metaclass
import warnings
from OCC.Wrapper.wrapper_utils import Proxy, deprecated

import OCC.Core.Standard
import OCC.Core.NCollection
import OCC.Core.AppParCurves
import OCC.Core.math
import OCC.Core.TColStd
import OCC.Core.TCollection
import OCC.Core.Message
import OCC.Core.OSD
import OCC.Core.gp
import OCC.Core.TColgp

from enum import IntEnum
from OCC.Core.Exception import *



class AppCont_Function(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    def D1(self, *args) -> "Standard_Boolean":
        r"""
        Returns the derivative at parameter <theu>.

        Parameters
        ----------
        theU: float
        theVec2d: NCollection_Array1<gp_Vec2d>
        theVec: NCollection_Array1<gp_Vec>

        Returns
        -------
        bool

        """
        return _AppCont.AppCont_Function_D1(self, *args)

    def FirstParameter(self, *args) -> "Standard_Real":
        r"""
        Returns the first parameter of the function.

        Returns
        -------
        float

        """
        return _AppCont.AppCont_Function_FirstParameter(self, *args)

    def GetNbOf2dPoints(self, *args) -> "Standard_Integer":
        r"""
        Get number of 2d points returned by 'value' and 'd1' functions.

        Returns
        -------
        int

        """
        return _AppCont.AppCont_Function_GetNbOf2dPoints(self, *args)

    def GetNbOf3dPoints(self, *args) -> "Standard_Integer":
        r"""
        Get number of 3d points returned by 'value' and 'd1' functions.

        Returns
        -------
        int

        """
        return _AppCont.AppCont_Function_GetNbOf3dPoints(self, *args)

    def GetNumberOfPoints(self, *args) -> "void":
        r"""
        Get number of 3d and 2d points returned by 'value' and 'd1' functions.

        Parameters
        ----------

        Returns
        -------
        theNbPnt: int
        theNbPnt2d: int

        """
        return _AppCont.AppCont_Function_GetNumberOfPoints(self, *args)

    def LastParameter(self, *args) -> "Standard_Real":
        r"""
        Returns the last parameter of the function.

        Returns
        -------
        float

        """
        return _AppCont.AppCont_Function_LastParameter(self, *args)

    def PeriodInformation(self, *args) -> "void":
        r"""
        Return information about peridicity in output paramateters space. @param thedimidx defines index in output parameters space. 1 <= thedimidx <= 3 * mynbpnt + 2 * mynbpnt2d.

        Parameters
        ----------
        Standard_Integer: 

        Returns
        -------
        IsPeriodic: bool
        thePeriod: float

        """
        return _AppCont.AppCont_Function_PeriodInformation(self, *args)

    def Value(self, *args) -> "Standard_Boolean":
        r"""
        Returns the point at parameter <theu>.

        Parameters
        ----------
        theU: float
        thePnt2d: NCollection_Array1<gp_Pnt2d>
        thePnt: NCollection_Array1<gp_Pnt>

        Returns
        -------
        bool

        """
        return _AppCont.AppCont_Function_Value(self, *args)

    __repr__ = _dumps_object

    __swig_destroy__ = _AppCont.delete_AppCont_Function

# Register AppCont_Function in _AppCont:
_AppCont.AppCont_Function_swigregister(AppCont_Function)

class AppCont_LeastSquare(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        No available documentation.

        Parameters
        ----------
        SSP: AppCont_Function
        U0: float
        U1: float
        FirstCons: AppParCurves_Constraint
        LastCons: AppParCurves_Constraint
        Deg: int
        NbPoints: int

        Returns
        -------
        None

        """
        _AppCont.AppCont_LeastSquare_swiginit(self, _AppCont.new_AppCont_LeastSquare(*args))

    def Error(self, *args) -> "void":
        r"""
        No available documentation.

        Parameters
        ----------

        Returns
        -------
        F: float
        MaxE3d: float
        MaxE2d: float

        """
        return _AppCont.AppCont_LeastSquare_Error(self, *args)

    def IsDone(self, *args) -> "Standard_Boolean":
        r"""
        No available documentation.

        Returns
        -------
        bool

        """
        return _AppCont.AppCont_LeastSquare_IsDone(self, *args)

    def Value(self, *args) -> "AppParCurves_MultiCurve const &":
        r"""
        No available documentation.

        Returns
        -------
        AppParCurves_MultiCurve

        """
        return _AppCont.AppCont_LeastSquare_Value(self, *args)

    __repr__ = _dumps_object

    __swig_destroy__ = _AppCont.delete_AppCont_LeastSquare

# Register AppCont_LeastSquare in _AppCont:
_AppCont.AppCont_LeastSquare_swigregister(AppCont_LeastSquare)





